/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;

import com.redhat.thermostat.common.tools.ApplicationException;
import com.redhat.thermostat.common.tools.ProcessStartException;

/**
 * Runs a process and logs it's output. The process's output and error streams
 * will be unreadable.
 */
public class LoggedExternalProcess extends Thread {

    private static final Logger logger = LoggingUtils.getLogger(LoggedExternalProcess.class);
    private BufferedReader reader;
    private String[] commands;

    public LoggedExternalProcess(List<String> commands) {
        this(commands.toArray(new String[0]));
    }
    
    public LoggedExternalProcess(String[] commands) {
        this.commands = Arrays.copyOf(commands, commands.length);
        setDaemon(true);
    }

    public int runAndReturnResult() throws IOException, InterruptedException, ApplicationException{
        Process p = runAndReturnProcess();
        return p.waitFor();
    }

    public Process runAndReturnProcess() throws IOException, ApplicationException {
        ProcessBuilder b = new ProcessBuilder(commands);
        b.redirectErrorStream(true);
        Process p = null;
        try {
            p = b.start();
        } catch (IOException ioe) {
            throw new ProcessStartException(commands[0], ioe);
        }
        reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
        this.start();
        return p;
    }

    @Override
    public void run() {
        try {
            String output = null;
            while ((output = reader.readLine()) != null) {
                logger.info(output);
            }
        } catch (IOException ex) {
            logger.warning("Error reading external program output.");
        } finally {
            try {
                reader.close();
            } catch (IOException ex) {
                logger.warning("Error closing stream from external program.");
            }
        }
    }
}

