/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.web.server.auth;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.security.Principal;
import java.security.acl.Group;
import java.util.Enumeration;
import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class WrappedRolePrincipalTest {

    private Group group;
    @SuppressWarnings("rawtypes")
    private Enumeration members = new Vector().elements();
    
    @SuppressWarnings("unchecked")
    @Before
    public void setup() {
        group = mock(Group.class);
        when(group.addMember(any(Principal.class))).thenReturn(true);
        when(group.isMember(any(Principal.class))).thenReturn(true);
        when(group.getName()).thenReturn("group-under-the-hood");
        when(group.members()).thenReturn(members);
        when(group.removeMember(any(Principal.class))).thenReturn(false);
    }
    
    @After
    public void tearDown() {
        group = null;
    }
    
    @Test
    public void testAllDelegates() {
        WrappedRolePrincipal p = new WrappedRolePrincipal(group);
        Principal mock = mock(Principal.class);
        assertEquals(true, p.addMember(mock));
        assertEquals(true, p.isMember(mock));
        assertEquals("group-under-the-hood", p.getName());
        assertEquals(members, p.members());
        assertEquals(false, p.removeMember(mock));
    }
}

