/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.client.core.experimental;

import com.redhat.thermostat.common.model.Range;

import java.util.List;

public class TimeRangeController <T, R> {

    public interface SingleArgRunnable <T> {
        public void run(T arg);
    }

    public interface StatsSupplier<T, R> {
        public abstract List<T> getStats(R ref, long since, long to);
    }

    private Range<Long> availableRange = new Range<>(Long.MAX_VALUE, Long.MIN_VALUE);
    private Range<Long> displayedRange = new Range<>(Long.MAX_VALUE, Long.MIN_VALUE);

    public void update(Duration userDesiredDuration, Range<Long> newAvailableRange, StatsSupplier<T, R> dao, R ref, SingleArgRunnable<T> updater) {
        long now = System.currentTimeMillis();
        long userVisibleTimeDelta = (userDesiredDuration.unit.toMillis(userDesiredDuration.value));
        Range<Long> desiredRange = new Range<>(now - userVisibleTimeDelta, now);

        if (availableRange.equals(newAvailableRange)) {
            return;
        }

        availableRange = newAvailableRange;

        List<Range<Long>> additionalIntervals = TimeRangeComputer.computeAdditionalRangesToFetch(availableRange, desiredRange, displayedRange);

        long displayedMin = Math.min(displayedRange.getMin(), Long.MAX_VALUE);
        long displayedMax = Math.max(displayedRange.getMax(), Long.MIN_VALUE);

        for (Range<Long> interval : additionalIntervals) {
            List<T> stats = dao.getStats(ref, interval.getMin(), interval.getMax());
            for (T stat : stats) {
                updater.run(stat);
            }

            displayedMin = Math.min(displayedMin, interval.getMin());
            displayedMax = Math.max(displayedMax, interval.getMax());
        }

        // the view does not show data older than the delta
        displayedMin = Math.max(displayedMin, displayedMax - userVisibleTimeDelta);

        displayedRange = new Range<>(displayedMin, displayedMax);
    }

    public Range<Long> getAvailableRange() {
        return availableRange;
    }
}
