/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.storage.internal.statement;

import java.util.Objects;

/**
 * 
 * Represents an {@link Unfinished} node in the where expressions parse tree
 * of prepared statements.
 * 
 * @see Patchable
 *
 */
class UnfinishedValueNode extends AbstractUnfinished {

    private int parameterIndex = -1;
    // determines if this patched value is a LHS or if false a RHS of a
    // binary comparison.
    private boolean isLHS;
    // Specifies the expected (component) type of this free parameter.
    private Class<?> type;

    Class<?> getType() {
        return type;
    }

    void setType(Class<?> type) {
        this.type = type;
    }

    boolean isLHS() {
        return isLHS;
    }

    void setLHS(boolean isLHS) {
        this.isLHS = isLHS;
    }

    @Override
    public int getParameterIndex() {
        return parameterIndex;
    }

    @Override
    public void setParameterIndex(int parameterIndex) {
        this.parameterIndex = parameterIndex;
    }

    @Override
    public String toString() {
        return "Unfinished value (" + getParameterIndex() + ") " + getType() +
                                 ":" + ( isLHS ? "LHS" : "RHS" );
    }
    
    @Override
    public boolean equals(Object other) {
        boolean basics = super.equals(other);
        if (!basics) {
            return false;
        }
        if (!(other instanceof UnfinishedValueNode)) {
            return false;
        }
        UnfinishedValueNode o = (UnfinishedValueNode)other;
        return basics && Objects.equals(isLHS(), o.isLHS) &&
                Objects.equals(getType(), o.getType());
    }
    
    @Override
    public int hashCode() {
        return Objects.hash(getParameterIndex(), isLHS(), getType());
    }

}

