/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.find.command.internal;

enum HostCriterion implements CriterionMatcher {
    HOSTNAME("hostname", new HostnameMatcher()),
    OS_KERNEL("oskernel", new OsKernelMatcher()),
    OS_NAME("osname", new OsNameMatcher()),
    ;

    private final String cliSwitch;
    private final CriterionMatcher criterionMatcher;

    HostCriterion(String cliSwitch, CriterionMatcher criterionMatcher) {
        this.cliSwitch = cliSwitch;
        this.criterionMatcher = criterionMatcher;
    }

    @Override
    public boolean match(MatchContext matchContext, String value) throws UnrecognizedArgumentException {
        return this.criterionMatcher.match(matchContext, value);
    }

    String getCliSwitch() {
        return cliSwitch;
    }

    static HostCriterion fromString(String string) {
        for (HostCriterion criterion : HostCriterion.values()) {
            if (criterion.cliSwitch.equals(string)) {
                return criterion;
            }
        }
        throw new IllegalArgumentException(string + " is not a legal HostCriterion");
    }

    static class HostnameMatcher implements CriterionMatcher {
        @Override
        public boolean match(MatchContext matchContext, String s) {
            return matchContext.getHostInfo().getHostname().equals(s);
        }
    }

    static class OsKernelMatcher implements CriterionMatcher {
        @Override
        public boolean match(MatchContext matchContext, String s) {
            String osKernel = matchContext.getHostInfo().getOsKernel();
            return osKernel.equals(s) || osKernel.contains(s);
        }
    }

    static class OsNameMatcher implements CriterionMatcher {
        @Override
        public boolean match(MatchContext matchContext, String s) {
            String osName = matchContext.getHostInfo().getOsName();
            return osName.equals(s) || osName.contains(s);
        }
    }


}
