/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.storage.query;

import java.util.Collections;
import java.util.Set;

/**
 * An {@link Expression} that contains a set of values.
 * @param <T> - the type of this expression's values
 */
public final class LiteralSetExpression<T> implements Expression {

    private Set<T> values;
    private Class<T> type;
    
    /**
     * Constructs a {@link LiteralSetExpression} given a list of values
     * and its type.
     * <p>
     * This constructor exists mainly for JSON serialization, use methods in
     * {@link ExpressionFactory} instead of this constructor.
     * @param values - a list of values for this expression
     * @param type - the type of values of stored in the provided set
     * (needed for serialization)
     */
    public LiteralSetExpression(Set<T> values, Class<T> type) {
        // Don't need to write, make read-only copy
        this.values = Collections.unmodifiableSet(values);
        this.type = type;
    }
    
    /**
     * @return the values represented by this expression
     */
    public Set<T> getValues() {
        return values;
    }
    
    /**
     * @return the type of values stored in this expression
     */
    public Class<T> getType() {
        return type;
    }
    
    @Override
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj != null && obj instanceof LiteralSetExpression) {
            LiteralSetExpression<?> otherExpr = (LiteralSetExpression<?>) obj;
            result = values.equals(otherExpr.values)
                    && type.equals(otherExpr.type);
        }
        return result;
    }
    
    @Override
    public int hashCode() {
        return values.hashCode() + type.hashCode();
    }
    
    @Override
    public String toString() {
        return values.toString();
    }
}

