/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.web.common.typeadapters;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.util.UUID;

import org.junit.Before;
import org.junit.Test;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.redhat.thermostat.web.common.SharedStateId;

public class SharedStateIdTypeAdapterTest {

private Gson gson;
    
    @Before
    public void setup() {
        gson = new GsonBuilder()
                    .registerTypeAdapter(SharedStateId.class, new SharedStateIdTypeAdapter())
                    .create();
    }
    
    @Test
    public void canSerializeBasic() {
        UUID uuid = UUID.randomUUID();
        SharedStateId id = new SharedStateId(3, uuid);
        String gsonStr = gson.toJson(id);
        String expectedJSON = "{\"sid\":3,\"stok\":\"" + uuid.toString() + "\"}";
        assertEquals(expectedJSON, gsonStr);
    }
    
    @Test
    public void canSerializeNull() {
        SharedStateId id = new SharedStateId(3000, null);
        String gsonStr = gson.toJson(id);
        String expectedJSON = "{\"sid\":3000}";
        assertEquals(expectedJSON, gsonStr);
    }
    
    @Test
    public void canDeserializeBasic() {
        UUID uuid = UUID.randomUUID();
        SharedStateId id = new SharedStateId(3, uuid);
        String json = "{\"sid\":3,\"stok\":\"" + uuid.toString() + "\"}";
        SharedStateId deserialized = gson.fromJson(json, SharedStateId.class);
        assertEquals(3, deserialized.getId());
        assertEquals(uuid, deserialized.getServerToken());
        assertEquals(id, deserialized);
    }
    
    /**
     * UUID string value might be null. Make sure deserialization works for
     * those.
     */
    @Test
    public void canDeserializeNullServerNonce() {
        String json = "{\"sid\":3}";
        SharedStateId deserialized = gson.fromJson(json, SharedStateId.class);
        assertEquals(3, deserialized.getId());
        assertNull(deserialized.getServerToken());
    }
}
