/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.storage.core.experimental.statement;

import java.util.HashMap;
import java.util.Map;

/**
 *
 */
public class TypeMapper {

    private static final Map<Class, String> map = new HashMap<>();
    static {
        map.put(String.class, "?s");
        map.put(int.class, "?i");
        map.put(long.class, "?l");
        map.put(boolean.class, "?b");
    }

    static String get(Class type) {
        return map.get(type);
    }

    static enum Symbol {

        QuerySeparator("AND"),
        InsertSeparator(","),

        Quote("'"),
        Space(" "),

        Sort("SORT"),
        Limit("LIMIT"),

        ;

        private final String definition;
        Symbol(String definition) {
            this.definition = definition;
        }

        public String id() {
            return definition;
        }
    }

    public static enum Criteria {
        Less("<"),
        LessEqual("<="),

        Equal("="),

        Greater(">"),
        GreaterEqual(">="),

        ;

        private final String definition;
        Criteria(String definition) {
            this.definition = definition;
        }

        public String id() {
            return definition;
        }
    }

    public static enum Sort {
        Ascending("ASC"),
        Descending("DSC"),
        ;

        private final String definition;
        Sort(String definition) {
            this.definition = definition;
        }

        public String id() {
            return definition;
        }
    }

    static enum Statement {
        Query,
        Insert,
    }

    static enum Clause {

        Add("ADD"),
        Set("SET"),

        Query("QUERY"),
        Where("WHERE"),

        ;

        private final String definition;
        Clause(String definition) {
            this.definition = definition;
        }

        public String id() {
            return definition;
        }
    }
}
