/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.web.common.typeadapters;

import java.io.IOException;

import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonToken;
import com.google.gson.stream.JsonWriter;
import com.redhat.thermostat.web.common.SharedStateId;
import com.redhat.thermostat.web.common.WebPreparedStatementResponse;

class WebPreparedStatementResponseTypeAdapter extends
        TypeAdapter<WebPreparedStatementResponse> {
    
    private static final String NUM_FREE_VARS_NAME = "numFreeVars";
    private static final String STMT_ID_NAME = "stmtId";
    private final TypeAdapter<SharedStateId> sharedStateTa;
    
    WebPreparedStatementResponseTypeAdapter(TypeAdapter<SharedStateId> sharedStateTa) {
        this.sharedStateTa = sharedStateTa;
    }
    
    @Override
    public WebPreparedStatementResponse read(JsonReader reader)
            throws IOException {
        if (reader.peek() == JsonToken.NULL) {
            reader.nextNull();
            return null;
        }
        
        WebPreparedStatementResponse response = new WebPreparedStatementResponse();
        
        reader.beginObject();
        String name = reader.nextName();
        if (name.equals(NUM_FREE_VARS_NAME)) {
            response.setNumFreeVariables(reader.nextInt());
        }
        name = reader.nextName();
        if (name.equals(STMT_ID_NAME)) {
            SharedStateId id = sharedStateTa.read(reader);
            response.setStatementId(id);
        }
        reader.endObject();
        
        return response;
    }

    @Override
    public void write(JsonWriter writer, WebPreparedStatementResponse value)
            throws IOException {
        if (value == null) {
            writer.nullValue();
            return;
        }
        
        int freeVars = value.getNumFreeVariables();
        SharedStateId stmtId = value.getStatementId();
        
        writer.beginObject();
        
        // Free variables
        writer.name(NUM_FREE_VARS_NAME);
        writer.value(freeVars);
        // stmt id
        writer.name(STMT_ID_NAME);
        sharedStateTa.write(writer, stmtId);

        writer.endObject();
    }

}
