/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.storage.internal.statement;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import com.redhat.thermostat.storage.core.Add;
import com.redhat.thermostat.storage.core.AggregateQuery;
import com.redhat.thermostat.storage.core.AggregateQuery.AggregateFunction;
import com.redhat.thermostat.storage.core.BackingStorage;
import com.redhat.thermostat.storage.core.Category;
import com.redhat.thermostat.storage.core.Query;
import com.redhat.thermostat.storage.core.Remove;
import com.redhat.thermostat.storage.core.Replace;
import com.redhat.thermostat.storage.core.StatementDescriptor;
import com.redhat.thermostat.storage.core.Update;
import com.redhat.thermostat.storage.model.Pojo;
import com.redhat.thermostat.storage.testutils.ParserType;

class DescriptorParserImplFactory {
    
    <T extends Pojo> StatementDescriptorParser<T> getParser(ParserType type, StatementDescriptor<T> desc) {
        switch (type) {
        case BASIC:
            return new BasicDescriptorParser<>(getBasicBackingStorage(), desc);
        case SEMANTIC:
            return new SemanticsEnabledDescriptorParser<>(getSemanticBackingStorage(), desc);
        default:
            throw new IllegalStateException("Not implemented");
        }
    }
    
    private BackingStorage getBasicBackingStorage() {
        return mock(BackingStorage.class);
    }
    
    @SuppressWarnings("unchecked")
    private BackingStorage getSemanticBackingStorage() {
        BackingStorage storage = mock(BackingStorage.class);
        when(storage.createAdd(any(Category.class))).thenReturn(mock(Add.class));
        when(storage.createUpdate(any(Category.class))).thenReturn(mock(Update.class));
        when(storage.createRemove(any(Category.class))).thenReturn(mock(Remove.class));
        when(storage.createReplace(any(Category.class))).thenReturn(mock(Replace.class));
        when(storage.createQuery(any(Category.class))).thenReturn(mock(Query.class));
        when(storage.createAggregateQuery(any(AggregateFunction.class), any(Category.class))).thenReturn(mock(AggregateQuery.class));
        return storage;
    }
}
