/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.shared.locale;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;

public class Translate<T extends Enum<T>> {

    private final ResourceBundle resourceBundle;

    public Translate(String stringResources, Class<T> enumClass) {
        this(ResourceBundle.getBundle(stringResources, Locale.getDefault(), enumClass.getClassLoader()), enumClass);
    }

    Translate(ResourceBundle resourceBundle, Class<T> enumClass) {
        this.resourceBundle = resourceBundle;
    }

    public LocalizedString localize(T toTranslate) {
        return new LocalizedString(resourceBundle.getString(toTranslate.name()));
    }

    public LocalizedString localize(T toTranslate, String... params) {
        return new LocalizedString(MessageFormat.format(localize(toTranslate).getContents(), (Object[]) params));
    }

    /**
     * Creates a {@link LocalizedString} that can contain one list of unknown length
     * that gets inserted as a single String, with separator as specified, into
     * the regular translate parameters at a given position.
     * @param toTranslate The translation key
     * @param specialParams The list to be inserted into the parameters
     * @param separator The separator to use between each element in the list
     * @param position The position within the regular parameters at which to insert the list-string (zero indexed)
     * @param params The parameters to apply to the translation
     * @return The {@link LocalizedString}
     */
    public LocalizedString localize(T toTranslate, String[] specialParams, String separator, int position, String... params) {
        StringBuilder builder = new StringBuilder(specialParams[0]);
        for (int i = 1; i < specialParams.length; i++) {
            builder.append(separator).append(specialParams[i]);
        }
        String replacedList = builder.toString();


        
        String[] newParams = new String[params.length + 1];
        for (int i = 0; i < position; i++) {
            newParams[i] = params[i];
        }
        newParams[position] = replacedList;
        for (int i = position + 1, j = position; j < params.length; i++, j++) {
            newParams[i] = params[j];
        }
        return new LocalizedString(MessageFormat.format(localize(toTranslate).getContents(), (Object[]) newParams));
    }
}

