/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.client.swing.internal.search;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import com.redhat.thermostat.storage.core.HostRef;
import com.redhat.thermostat.storage.core.Ref;
import com.redhat.thermostat.storage.core.VmRef;

/**
 *
 */
class SearchBackend {
    
    private HashMap<HostRef, Set<VmRef>> hosts;
    
    public SearchBackend() {
        hosts = new HashMap<>();
    }
    
    public synchronized boolean match(String pattern, Ref reference) {

        boolean match = false;
        
        String searchString = pattern.toLowerCase();
        
        String name = reference.getName().toLowerCase();
        String id = reference.getStringID().toLowerCase();
        
        match = name.contains(searchString) || id.contains(searchString);
        if (!match && (reference instanceof HostRef)) {
            HostRef host = (HostRef) reference;
            if (hosts.containsKey(host)) {
                for (VmRef vm : hosts.get(host)) {
                    match = match(searchString, vm);
                
                    // since this is an host, no need to do all, as long as
                    // we know there's at least one match, this host will
                    // show up in the reference tree
                    if (match) {
                        break;
                    }
                }
            }
        }
        
        return match;
    }
    
    public synchronized void addVM(VmRef vm) {
        HostRef ref = vm.getHostRef();
        hosts.get(ref).add(vm);
    }

    public synchronized void addVMs(HostRef host, Collection<VmRef> vms) {
        hosts.get(host).addAll(vms);
    }

    public synchronized void addHost(HostRef host) {
        hosts.put(host, new HashSet<VmRef>());
    }

    public synchronized void addHosts(Collection<HostRef> hosts) {
        for (HostRef host : hosts) {
            addHost(host);
        }
    }
}

