/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.launcher.internal;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;

import com.redhat.thermostat.launcher.BundleInformation;

public class TestCommandInfo implements CommandInfo {

    private String name;
    private String summary;
    private String description;
    private String usage;

    private Options options = new Options();
    private Set<Environment> environments;
    private boolean fileTabCompletionNeeded;

    public TestCommandInfo(String name) {
        this.name = name;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getSummary() {
        return summary;
    }

    public void setSummary(String summary) {
        this.summary = summary;
    }

    @Override
    public String getDescription() {
        return description;
    }

    public void setDescription(String desc) {
        description = desc;
    }

    @Override
    public String getUsage() {
        return usage;
    }

    public void setUsage(String usage) {
        this.usage = usage;
    }

    @Override
    public Options getOptions() {
        return options;
    }

    @Override
    public boolean needsFileTabCompletions() {
        return fileTabCompletionNeeded;
    }

    public void setFileTabCompletionNeeded(boolean fileTabCompletionNeeded) {
        this.fileTabCompletionNeeded = fileTabCompletionNeeded;
    }

    public void addOptions(Option... options) {
        for (Option option : options) {
            this.options.addOption(option);
        }
    }

    @Override
    public Set<Environment> getEnvironments() {
        return environments;
    }

    public void setAvailable(Set<Environment> environments) {
        this.environments = environments;
    }

    @Override
    public List<BundleInformation> getBundles() {
        return Collections.emptyList();
    }
}

